<?php

class Shop_Payment_System_Handler<№> extends Shop_Payment_System_Handler
{
    /**
     * Идентификатор валюты, в которой передается сумма в платежную систему
     */
    protected $_currency = 1;
	
		/**
     * URL сервера PayKeeper
     */
    private $_ServerURL = "http://<inter_your_site_name_here>.server.paykeeper.ru/create/";
	
		/**
     * Секретное слово
     */
    private $_Secret = "the_secret_word";

    /**
     * Метод, вызываемый в коде настроек ТДС через Shop_Payment_System_Handler::checkBeforeContent($oShop); Может быть использован для получения данных от платежной системе о статусе платежа.
     */
    public function checkPaymentBeforeContent()
    {
				if (isset($_POST['id']) && isset($_POST['key']) && isset($_POST['sum']) && isset($_POST['orderid']) && isset($_POST['clientid']))
        {
						// Получаем ID заказа
            $order_id = intval(Core_Array::getRequest('orderid'));

            $oShop_Order = Core_Entity::factory('Shop_Order')->find($order_id);

            if (!is_null($oShop_Order->id))
            {
                // Вызов обработчика платежной системы
                Shop_Payment_System_Handler::factory($oShop_Order->Shop_Payment_System)
                    ->shopOrder($oShop_Order)
                    ->paymentProcessing();
            }
            exit;
        }
    }

    /**
     * Метод, вызываемый в коде ТДС через Shop_Payment_System_Handler::checkAfterContent($oShop); Может быть использован как для получения информации от платежной системы о статусе платежа, так и о выводе информации о результатах оплаты после перенаправления пользователя платежной системой на корзину магазина.
     */
    public function checkPaymentAfterContent()
    {
        if (isset($_REQUEST['SignatureValue']))
        {
            // Получаем ID заказа
            $order_id = intval(Core_Array::getRequest('InvId'));

            $oShop_Order = Core_Entity::factory('Shop_Order')->find($order_id);

            if (!is_null($oShop_Order->id))
            {
                // Вызов обработчика платежной системы
                Shop_Payment_System_Handler::factory($oShop_Order->Shop_Payment_System)
                    ->shopOrder($oShop_Order)
                    ->paymentProcessing();
            }
        }
    }



    /*
     * Метод, запускающий выполнение обработчика
     */
    public function execute()
    {
        parent::execute();

        // Показ формы оплаты (перехода на оплату), сама форма генерируется в getNotification()
        $this->printNotification();

        return $this;
    }

    /**
     * Оформление нового заказа
     */
    protected function _processOrder()
    {
        // Вызываем стандартное оформление заказа из родительского класса Shop_Payment_System_Handler
        parent::_processOrder();

        // Установка XSL-шаблонов в соответствии с настройками в узле структуры
        $this->setXSLs();

        // Отправка писем клиенту и пользователю
        $this->send();

        return $this;
    }

    /**
     * Обработка уведомлений об оплате заказа или возврате пользователя из магазина.
     * Используются уникальные параметры, передаваемые платежной системой и позволяющие
     * определить, идет уведомление об платежной системы об оплате или возврат пользователя
     * из платежной системы на сайт после успешной/неуспешной оплаты.
     */
    public function paymentProcessing()
    {
        // Пользователь перешел на страницу с уведомлением о статусе заказа
        /*
				if (isset($_REQUEST['InvId']) && isset($_REQUEST['некийПараметр']))
        {
            $this->showResultMessage();
            return TRUE;
        }
				*/

        // Пришло подтверждение оплаты, обработаем его
        if (isset($_REQUEST['orderid']))
        {
            $this->ProcessResult();
            return TRUE;
        }
    }

    /**
     * Сумма заказа в валюте платежной системы ($this->_currency)
     */
    public function getSumWithCoeff()
    {
        return Shop_Controller::instance()->round(($this->_currency > 0
                && $this->_shopOrder->shop_currency_id > 0
            ? Shop_Controller::instance()->getCurrencyCoefficientInShopCurrency(
                $this->_shopOrder->Shop_Currency,
                Core_Entity::factory('Shop_Currency', $this->_currency)
            )
            : 0) * $this->_shopOrder->getAmount() );
    }

    /**
     * Возвращает строку с формой перехода к оплате
     */
    public function getNotification()
    {
      $sum = $this->getSumWithCoeff();
				
			if ($this->_shopOrder->name != NULL || $this->_shopOrder->surname != NULL || $this->_shopOrder->patronymic != NULL) {
				$clientid = $this->_shopOrder->name . " " . $this->_shopOrder->surname . " " . $this->_shopOrder->patronymic;
			}
			elseif ($this->_shopOrder->email != NULL) {
				$clientid = $this->_shopOrder->email;
			}
			else {
				$clientid = "Guest";
			}
			
			$orderid = $this->_shopOrder->id;
			
			$client_email = $this->_shopOrder->email;
			
			$client_phone = $this->_shopOrder->phone;
			
			$service_name = $this->_shopOrder->description;
			
			$to_hash      = number_format ($sum , 2 , '.' , '') . 
                                     $clientid            .   
                                     $orderid             .   
                                     $service_name        .   
                                     $client_email        .   
                                     $client_phone        .   
                                     $this->_Secret;

			$sign = hash ('sha256' , $to_hash);

        ob_start();
        ?>
        <h1>Оплата заказа № <?php echo $this->_shopOrder->id?> через платежную платформу PayKeeper</h1>

        <p>Сумма к оплате составляет <strong><?php echo $sum?> <?php echo Core_Entity::factory('Shop_Currency', $this->_currency)->name?></strong></p>

        <p>Для оплаты нажмите кнопку "Оплатить".</p>

        <form action="<?php echo $this->_ServerURL?>" method="POST">
            <input type="hidden" name="sum" value="<?php echo $sum?>">
			<input type="hidden" name="clientid" value="<?php echo $clientid?>">
            <input type="hidden" name="orderid" value="<?php echo $orderid?>">
            <input type="hidden" name="client_email" value="<?php echo $client_email?>">
			<input type="hidden" name="client_phone" value="<?php echo $client_phone?>">
			<input type="hidden" name="service_name" value="<?php echo $service_name?>">
			<input type="hidden" name="sign" value="<?php echo $sign?>">
            <input type="submit" value="Оплатить" class="cart_button">
        </form>
        <?php

        return ob_get_clean();
    }

    /**
     * Получить код счета (в данном случае равен коду перехода на оплату).
     * Используется, например, в центре администрирования при печати заказа
     */
    public function getInvoice()
    {
        return $this->getNotification();
    }

    // Вывод сообщения об успешности/неуспешности оплаты
    function showResultMessage()
    {
        $iShop_Order_Id = Core_Array::getRequest('InvId', 0);
        $oShop_Order = Core_Entity::factory('Shop_Order')->find($iShop_Order_Id);

        if(is_null($oShop_Order->id))
        {
            // Заказ не найден
            return FALSE;
        }

        $sHash = Core_Array::getRequest('Signature', '');
        $sSum = Core_Array::getRequest('Sum', '');

        // В данном примере сумма обязательно должна быть с двумя десятичными нулями
        $sHostcmsSum = sprintf("%.2f", $this->getSumWithCoeff());

        // Суммы совпали?
        $sHostcmsHash = $sSum == $sHostcmsSum
            ? md5("параметры_используемые_при_контроле_хэша")
            : '';

        // Сравниваем хэши
        if (mb_strtoupper($sHostcmsHash) == mb_strtoupper($sHash))
        {
            $sStatus = $oShop_Order->paid == 1 ? "оплачен" : "не оплачен";

            ?><h1>Заказ <?php echo $sStatus?></h1>
            <p>Заказ <strong>№ <?php echo $oShop_Order->invoice?></strong> <?php echo $sStatus?>.</p>
            <?php
        }
        else
        {
            ?><p>Хэш не совпал!</p><?php
        }
    }

    /*
     * Обработка статуса оплаты
    */
    function ProcessResult()
    {
        $iShop_Order_Id = Core_Array::getRequest('orderid', 0);
        $oShop_Order = Core_Entity::factory('Shop_Order')->find($iShop_Order_Id);

        if(is_null($oShop_Order->id) || $oShop_Order->paid)
        {
            // Заказ не найден
						echo "ERROR! Order not found.";
            return FALSE;
        }
				
		$id         = $_POST['id'];
    	$key      = $_POST['key'];
    	$sum     = $_POST['sum'];
    	$orderid = $_POST['orderid'];
    	$clientid = $_POST['clientid'];
					
		$hash = md5 ($id . number_format ($sum , 2 , '.' , '') . $clientid . $orderid . $this->_Secret);
							
        // В данном примере сумма обязательно должна быть с двумя десятичными нулями
        $sHostcmsSum = number_format($this->getSumWithCoeff() , 2 , '.' , '');
		$pSum = number_format ($sum , 2 , '.' , '');

        if ($pSum == $sHostcmsSum)
        {
            

            // Сравниваем хэши
            if ($key == $hash)
            {
                $this->shopOrder($oShop_Order)->shopOrderBeforeAction(clone $oShop_Order);

                $oShop_Order->system_information = "Товар оплачен через платежную платформу PayKeeper.\n";
                $oShop_Order->paid();
                $this->setXSLs();
                $this->send();

                // Ответ платежной системе на уведомление об оплате, см. API платежной системы
                echo "OK " . md5 ($id.$this->_Secret);

                ob_start();
                $this->changedOrder('changeStatusPaid');
                ob_get_clean();
            }
            else
            {
                $oShop_Order->system_information = 'PayKeeper: Error! Hash mismatch!';
                $oShop_Order->save();

                // Ответ платежной системе на уведомление об оплате, см. API платежной системы
                echo "ERROR! Hash mismatch!";
            }
        }
        else
        {
            $oShop_Order->system_information = 'PayKeeper: Error! Sum mismatch!';
            $oShop_Order->save();

            // Ответ платежной системе на уведомление об оплате, см. API платежной системы
            echo "ERROR! Sum mismatch!";
        }
    }
}